
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripLoader()
{
    this.hasWhitelist = false;
    this.hasBlacklist = false;
    this.hasStyles = false;

    this.load = function(doc)
    {
        if (!doc) return false;

        var pageAddress = yarip.getAddress(doc.location, yarip.PAGE_MODE);
        if (!pageAddress) return false;

        var aAddress = yarip.getAddressArray(pageAddress, true);
        this.doWhitelisting(doc, pageAddress, aAddress);
        this.doBlacklisting(doc, pageAddress, aAddress);
        this.doStyling(doc, pageAddress, aAddress);
        this.doExtensioning(doc, pageAddress, aAddress);
        this.doScripting(doc, pageAddress, aAddress);

        if (this.hasWhitelist && !this.hasBlacklist) yarip.removeAllExceptWhitelisted(doc, pageAddress);

        var found = this.hasWhitelist || this.hasBlacklist || this.hasStyles;
        this.hasWhitelist = false;
        this.hasBlacklist = false;
        this.hasStyles = false;
        return found;
    }

    this.doWhitelisting = function(doc, pageAddress, aAddress)
    {
        for (var a in aAddress)
        {
            if (!aAddress[a][1]) continue;
            var list = yarip.map.obj[a].elementWhitelist.obj;
            var equalAddresses = yarip.equalAddresses(pageAddress, a);
            for (var k in list)
            {
                var element = list[k];
                if (!element.getDeep() && !equalAddresses) continue;
                if (yarip.whitelistXpath(doc, a, element.getXPath(), false, null, aAddress[a][0])) this.hasWhitelist = true;
            }
        }
    }

    this.doBlacklisting = function(doc, pageAddress, aAddress)
    {
        for (var a in aAddress)
        {
            if (!aAddress[a][1]) continue;
            var list = yarip.map.obj[a].elementBlacklist.obj;
            var equalAddresses = yarip.equalAddresses(pageAddress, a);
            for (var k in list)
            {
                var element = list[k];
//                if (!element.getForce() && !element.getDeep() && !equalAddresses) continue;
                if (!element.getDeep() && !equalAddresses) continue;
                if (!this.hasWhitelist && element.getDOW()) continue;
                if (yarip.blacklistXpath(doc, a, element.getXPath(), false, element.getForce(), null, aAddress[a][0])) this.hasBlacklist = true;
            }
        }
    }

    this.doStyling = function(doc, pageAddress, aAddress)
    {
        for (var a in aAddress)
        {
            if (!aAddress[a][1]) continue;
            var list = yarip.map.obj[a].attributes.obj;
            var equalAddresses = yarip.equalAddresses(pageAddress, a);
            for (var k in list)
            {
                var element = list[k];
                if (!element.getDeep() && !equalAddresses) continue;
                if (yarip.setAttribute(doc, a, element.getXPath(), element.getName(), element.getValue(), null, null, aAddress[a][0])) this.hasStyles = true;
            }
        }
    }

    this.doExtensioning = function(doc, pageAddress, aAddress)
    {
        var aExtAddress = yarip.getExtensionAddressArray(aAddress);
        if (!aExtAddress) return;

        this.doWhitelisting(doc, pageAddress, aExtAddress);
        this.doBlacklisting(doc, pageAddress, aExtAddress);
        this.doStyling(doc, pageAddress, aExtAddress);
    }

    this.doScripting = function(doc, pageAddress, aAddress)
    {
        var tmp = [];
        for (var a in aAddress) {
            if (!aAddress[a][3]) {
                continue;
            } else {
                tmp[tmp.length] = a;
            }
        }
        for (var i = tmp.length; i > 0; i--)
        {
            var a = tmp[i-1];
            // TODO Move to nsYarip!
            var list = yarip.map.obj[a].script;
            if (list.length > 0)
            {
                var scriptId = "yarip-script_" + a.replace(/\W/g, "_");
                var script = doc.getElementById(scriptId);
                if (script && script.localName != "SCRIPT") {
                    script.parentNode.removeChild(script);
                    script = null;
                }

                if (!script) {
                    script = doc.createElement("script");
                    script.type = "text/javascript";
                    script.setAttribute("id", scriptId);
                    script.setAttribute("status", "whitelisted");
                    script.innerHTML = list.get();
                    doc.getElementsByTagName("head")[0].appendChild(script);
                } else {
                    script.type = "text/javascript";
                    script.setAttribute("status", "whitelisted");
                    script.innerHTML = list.get();
                }
            }
        }
    }
}
