
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

//function YaripElement(xpath, deep, force, dow, style, found, notFound, ignored, lastFound)
function YaripElement(xpath, deep, force, dow, style, lastFound, found, notFound, ignored)
{
    this.xpath = "";
    this.deep = true;
    this.force = true;
    this.dow = false;
    this.style = null;
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;
    this.ignored = 0;

    this.getKey = function()
    {
        return this.getXPath();
    }

    this.setXPath = function(value)
    {
        if (value) this.xpath = "" + value;
    }

    this.getXPath = function()
    {
        return this.xpath;
    }

    this.setDeep = function(value, noStyle)
    {
        this.deep = "" + value != "false";
        if (!this.deep && !noStyle) this.style = null;
    }

    this.getDeep = function()
    {
        return this.deep;
    }

    this.setForce = function(value)
    {
        this.force = "" + value != "false";
        if (!this.force) this.style = null;
    }

    this.getForce = function()
    {
        return this.force;
    }

    this.setDOW = function(value)
    {
        this.dow = "" + value == "true";
    }

    this.getDOW = function()
    {
        return this.dow;
    }

    this.setStyle = function(value)
    {
        if (value) this.style = value;
    }

    this.getStyle = function()
    {
        return this.style;
    }

    this.setLastFound = function(value)
    {
        if (!value) return;
        value = Number(value);
        if (typeof value != "number" || value + "" == "NaN") return;
        this.lastFound = value;
    }

    this.getLastFound = function()
    {
        return this.lastFound;
    }

    this.setFound = function(value)
    {
        if (!value) return;
        value = Number(value);
        if (typeof value != "number" || value + "" == "NaN") return;
        this.found = value;
    }

    this.getFound = function()
    {
        return this.found;
    }

    this.incrementFound = function()
    {
        this.found++;
        this.lastFound = new Date().getTime();
    }

    this.setNotFound = function(value)
    {
        if (!value) return;
        value = Number(value);
        if (typeof value != "number" || value + "" == "NaN") return;
        this.notFound = value;
    }

    this.getNotFound = function()
    {
        return this.notFound;
    }

    this.incrementNotFound = function()
    {
        this.notFound++;
    }

    this.setIgnored = function(value)
    {
        if (!value) return;
        value = Number(value);
        if (typeof value != "number" || value + "" == "NaN") return;
        this.ignored = value;
    }

    this.getIgnored = function()
    {
        return this.ignored;
    }

    this.incrementIgnored = function()
    {
        this.ignored++;
    }

    this.purge = function()
    {
        this.lastFound = -1;
        this.found = 0;
        this.notFound = 0;
        this.ignored = 0;
    }

    this.clone = function(purge)
    {
        var tmp = new this.constructor(this.xpath, this.deep, this.force, this.dow, this.style, this.lastFound, this.found, this.notFound, this.ignored);
        for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(xpath|deep|force|dow|style|lastFound|found|notFound|ignored)$/.test(k)) tmp[k] = this[k];
        if (purge) tmp.purge();
        return tmp;
    }

    this.merge = function(o)
    {
        if (!o) return;
        this.setDeep(o.getDeep() && this.getDeep());
        this.setForce(o.getForce() && this.getForce());
        this.setDOW(o.getDOW() || this.getDOW());
        this.setStyle(o.getStyle() || this.getStyle());
//        var t1 = o.getLastFound();
//        var t2 = this.getLastFound();
//        this.setLastFound(t1 > t2 ? t1 : t1 < t2 ? t2 : -1);
//        this.setFound(o.getFound() + this.getFound());
//        this.setNotFound(o.getNotFound() + this.getNotFound());
//        this.setIgnored(o.getIgnored() + this.getIgnored());
    }

    this.generateXml = function()
    {
        if (!this.xpath) return "";
        return "\t\t\t" +
            "<element" +
                " deep=\"" + this.deep + "\"" +
                " force=\"" + this.force + "\"" +
                " dow=\"" + this.dow + "\"" +
                (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
                (this.found > 0 ? " found=\"" + this.found + "\"" : "") +
                (this.notFound > 0 ? " notFound=\"" + this.notFound + "\"" : "") +
                (this.ignored > 0 ? " ignored=\"" + this.ignored + "\"" : "") +
            ">\n" +
            "\t\t\t\t<xpath><![CDATA[" + this.xpath + "]]></xpath>\n" +
            (this.style ? "\t\t\t\t<style><![CDATA[" + this.style + "]]></style>\n" : "") +
            "\t\t\t</element>\n";
    }

    this.createStyle = function(force)
    {
        if (!this.force || !force && this.style) return;
        this.style = null;
        if (/*!this.deep ||*/ !/^\/?(\/([a-z][\w]*|\*)(\[@[a-z][\w]*='[^']*'\])*)+$/i.test(this.xpath)) return;
        this.style = this.xpath.replace(/^\/+/, "").replace(/(([a-z][\w]*|\*)(\[@[a-z][\w]*='[^']*'\])*)\//gi, "$1 > ").replace(/\[@/g, "\[");
    }

    this.xpathToStyle = function()
    {
        if (!this.force || !/^\/?(\/([a-z][\w]*|\*)(\[@[a-z][\w]*='[^']*'\])*)+$/i.test(this.xpath)) return null;
        return this.xpath.replace(/^\/+/, "").replace(/(([a-z][\w]*|\*)(\[@[a-z][\w]*='[^']*'\])*)\//gi, "$1 > ").replace(/\[@/g, "\[");
    }

    this.generateCSS = function(notDeep)
    {
//        if (notDeep && this.deep) return "";
        if (!notDeep == this.deep && this.force && this.style) return "\t" + this.style + ",\n";
        else return "";
    }

    this.setXPath(xpath);
    this.setDeep(deep);
    this.setForce(force);
    this.setDOW(dow);
    this.setStyle(style);
    this.setLastFound(lastFound);
    this.setFound(found);
    this.setNotFound(notFound);
    this.setIgnored(ignored);
}

function YaripWLElement(xpath, deep, lastFound, found, notFound, ignored)
{
    this.xpath = "";
    this.deep = true;
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;
    this.ignored = 0;

    this.setXPath(xpath);
    this.setDeep(deep);
    this.setLastFound(lastFound);
    this.setFound(found);
    this.setNotFound(notFound);
    this.setIgnored(ignored);
}
YaripWLElement.prototype = new YaripElement;
YaripWLElement.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.xpath, this.deep, this.lastFound, this.found, this.notFound, this.ignored);
    for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(xpath|deep|lastFound|found|notFound|ignored)$/.test(k)) tmp[k] = this[k];
    if (purge) tmp.purge();
    return tmp;
}
YaripWLElement.prototype.merge = function(o)
{
    if (!o) return;
    this.setDeep(o.getDeep() && this.getDeep());
//    var t1 = o.getLastFound();
//    var t2 = this.getLastFound();
//    this.setLastFound(t1 > t2 ? t1 : t1 < t2 ? t2 : -1);
//    this.setFound(o.getFound() + this.getFound());
//    this.setNotFound(o.getNotFound() + this.getNotFound());
//    this.setIgnored(o.getIgnored() + this.getIgnored());
}
YaripWLElement.prototype.generateXml = function()
{
    if (!this.xpath) return "";
    return "\t\t\t" +
        "<element" +
            " deep=\"" + this.deep + "\"" +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
            (this.found > 0 ? " found=\"" + this.found + "\"" : "") +
            (this.notFound > 0 ? " notFound=\"" + this.notFound + "\"" : "") +
            (this.ignored > 0 ? " ignored=\"" + this.ignored + "\"" : "") +
        ">\n" +
        "\t\t\t\t<xpath><![CDATA[" + this.xpath + "]]></xpath>\n" +
        "\t\t\t</element>\n";
}

function YaripBLElement(xpath, deep, force, dow, style, lastFound, found, notFound, ignored)
{
    this.xpath = "";
    this.deep = true;
    this.force = true;
    this.dow = false;
    this.style = null;
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;
    this.ignored = 0;

    this.setXPath(xpath);
    this.setDeep(deep);
    this.setForce(force);
    this.setDOW(dow);
    this.setStyle(style);
    this.setLastFound(lastFound);
    this.setFound(found);
    this.setNotFound(notFound);
    this.setIgnored(ignored);
}
YaripBLElement.prototype = new YaripElement;

function YaripContent(regexp, deep, force, lastFound, ignored)
{
    this.regexp = "";
    this.regexpobj = null;
    this.deep = true;
    this.force = true;
    this.lastFound = -1;
    this.ignored = 0;

    this.getKey = function()
    {
        return this.getRegExp();
    }

    this.setRegExp = function(value)
    {
        if (!value) return;

        this.regexp = value;
        this.setRegExpObj(value);
    }

    this.getRegExp = function()
    {
        return this.regexp;
    }

    this.setRegExpObj = function(value)
    {
        if (value) this.regexpobj = new RegExp(value);
    }

    this.getRegExpObj = function()
    {
        return this.regexpobj;
    }

    this.setDeep = function(value)
    {
        this.deep = "" + value != "false";
    }

    this.getDeep = function()
    {
        return this.deep;
    }

    this.setForce = function(value)
    {
        this.force = "" + value != "false";
    }

    this.getForce = function()
    {
        return this.force;
    }

    this.setLastFound = function(value)
    {
        if (!value) return;
        value = Number(value);
        if (typeof value != "number" || value + "" == "NaN") return;
        this.lastFound = value;
    }

    this.getLastFound = function()
    {
        return this.lastFound;
    }

    this.incrementLastFound = function()
    {
        this.lastFound = new Date().getTime();
    }

    this.setIgnored = function(value)
    {
        if (!value) return;
        value = Number(value);
        if (typeof value != "number" || value + "" == "NaN") return;
        this.ignored = value;
    }

    this.getIgnored = function()
    {
        return this.ignored;
    }

    this.incrementIgnored = function()
    {
        this.ignored++;
    }

    this.purge = function()
    {
        this.lastFound = -1;
        this.ignored = 0;
    }

    this.clone = function(purge)
    {
        var tmp = new this.constructor(this.regexp, this.deep, this.force, this.lastFound, this.ignored);
        for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(regexp|deep|force|lastFound|ignored)$/.test(k)) tmp[k] = this[k];
        if (purge) tmp.purge();
        return tmp;
    }

    this.merge = function(o)
    {
        if (!o) return;
        this.setRegExp(o.getRegExp() || this.getRegExp());
        this.setDeep(o.getDeep() && this.getDeep());
        this.setForce(o.getForce() && this.getForce());
//        var t1 = o.getLastFound();
//        var t2 = this.getLastFound();
//        this.setLastFound(t1 > t2 ? t1 : t1 < t2 ? t2 : -1);
//        this.setIgnored(o.getIgnored() + this.getIgnored());
    }

    this.generateXml = function()
    {
        if (!this.regexp) return "";
        return "\t\t\t" +
            "<content" +
                " deep=\"" + this.deep + "\"" +
                " force=\"" + this.force + "\"" +
                (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
                (this.ignored > 0 ? " ignored=\"" + this.ignored + "\"" : "") +
            ">\n" +
            "\t\t\t\t<regexp><![CDATA[" + this.regexp + "]]></regexp>\n" +
            "\t\t\t</content>\n";
    }

    this.setRegExp(regexp);
    this.setDeep(deep);
    this.setForce(force);
    this.setLastFound(lastFound);
    this.setIgnored(ignored);
}

function YaripWLContent(regexp, deep, lastFound)
{
    this.regexp = "";
    this.regexpobj = null;
    this.deep = true;
    this.lastFound = -1;

    this.setRegExp(regexp);
    this.setDeep(deep);
    this.setLastFound(lastFound);
}
YaripWLContent.prototype = new YaripContent;
YaripWLContent.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.regexp, this.deep, this.lastFound);
    for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(regexp|deep|lastFound)$/.test(k)) tmp[k] = this[k];
    if (purge) tmp.purge();
    return tmp;
}
YaripWLContent.prototype.merge = function(o)
{
    if (!o) return;
    this.setRegExp(o.getRegExp() || this.getRegExp());
    this.setDeep(o.getDeep() && this.getDeep());
//    var t1 = o.getLastFound();
//    var t2 = this.getLastFound();
//    this.setLastFound(t1 > t2 ? t1 : t1 < t2 ? t2 : -1);
}
YaripWLContent.prototype.generateXml = function()
{
    if (!this.regexp) return "";
    return "\t\t\t" +
        "<content" +
            " deep=\"" + this.deep + "\"" +
            (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
        ">\n" +
        "\t\t\t\t<regexp><![CDATA[" + this.regexp + "]]></regexp>\n" +
        "\t\t\t</content>\n";
}

function YaripBLContent(regexp, deep, force, lastFound, ignored)
{
    this.regexp = "";
    this.regexpobj = null;
    this.deep = true;
    this.force = true;
    this.lastFound = -1;
    this.ignored = 0;

    this.setRegExp(regexp);
    this.setDeep(deep);
    this.setForce(force);
    this.setLastFound(lastFound);
    this.setIgnored(ignored);
}
YaripBLContent.prototype = new YaripContent;

function YaripAttribute(xpath, name, value, deep, lastFound, found, notFound)
{
    this.xpath = "";
    this.name = null;
    this.value = "";
    this.deep = true;
    this.lastFound = -1;
    this.found = 0;
    this.notFound = 0;

    this.getKey = function()
    {
//        return this.getXPath();
        return this.getXPath() + this.getName();
    }

    this.setXPath = function(value)
    {
        if (value) this.xpath = "" + value;
    }

    this.getXPath = function()
    {
        return this.xpath;
    }

    this.setName = function(value)
    {
        if (value) this.name = value;
    }

    this.getName = function()
    {
        return this.name;
    }

    this.setValue = function(value)
    {
        if (value) this.value = value;
    }

    this.getValue = function()
    {
        return this.value;
    }

    this.setDeep = function(value, noStyle)
    {
        this.deep = "" + value != "false";
        if (!this.deep && !noStyle) this.style = null;
    }

    this.getDeep = function()
    {
        return this.deep;
    }

    this.setLastFound = function(value)
    {
        if (!value) return;
        value = Number(value);
        if (typeof value != "number" || value + "" == "NaN") return;
        this.lastFound = value;
    }

    this.getLastFound = function()
    {
        return this.lastFound;
    }

    this.setFound = function(value)
    {
        if (!value) return;
        value = Number(value);
        if (typeof value != "number" || value + "" == "NaN") return;
        this.found = value;
    }

    this.getFound = function()
    {
        return this.found;
    }

    this.incrementFound = function()
    {
        this.found++;
        this.lastFound = new Date().getTime();
    }

    this.setNotFound = function(value)
    {
        if (!value) return;
        value = Number(value);
        if (typeof value != "number" || value + "" == "NaN") return;
        this.notFound = value;
    }

    this.getNotFound = function()
    {
        return this.notFound;
    }

    this.incrementNotFound = function()
    {
        this.notFound++;
    }

    this.purge = function()
    {
        this.lastFound = -1;
        this.found = 0;
        this.notFound = 0;
    }

    this.clone = function(purge)
    {
        var tmp = new this.constructor(this.xpath, this.name, this.value, this.deep, this.lastFound, this.found, this.notFound);
        for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(xpath|name|value|deep|lastFound|found|notFound)$/.test(k)) tmp[k] = this[k];
        if (purge) tmp.purge();
        return tmp;
    }

    this.merge = function(o)
    {
        if (!o) return;
        this.setName(o.getName() || this.getName());
        this.setValue(o.getValue() || this.getValue());
        this.setDeep(o.getDeep() && this.getDeep());
//        var t1 = o.getLastFound();
//        var t2 = this.getLastFound();
//        this.setLastFound(t1 > t2 ? t1 : t1 < t2 ? t2 : -1);
//        this.setFound(o.getFound() + this.getFound());
//        this.setNotFound(o.getNotFound() + this.getNotFound());
    }

    this.generateXml = function()
    {
        if (!this.xpath || !this.name || !this.value) return "";
        var tmp = "\t\t\t" +
            "<element" +
                " deep=\"" + this.deep + "\"" +
                (this.lastFound > -1 ? " lastFound=\"" + this.lastFound + "\"" : "") +
                (this.found > 0 ? " found=\"" + this.found + "\"" : "") +
                (this.notFound > 0 ? " notFound=\"" + this.notFound + "\"" : "") +
            ">\n" +
            "\t\t\t\t<xpath><![CDATA[" + this.xpath + "]]></xpath>\n" +
            "\t\t\t\t<name><![CDATA[" + this.name + "]]></name>\n" +
            "\t\t\t\t<value><![CDATA[" + this.value + "]]></value>\n" +
            "\t\t\t</element>\n";
        return tmp;
    }

    this.setXPath(xpath);
    this.setName(name);
    this.setValue(value);
    this.setDeep(deep);
    this.setLastFound(lastFound);
    this.setFound(found);
    this.setNotFound(notFound);
}

function YaripItem(value, doElements, doContents, doScripts)
{
    this.value = "";
    this.doElements = false;
    this.doContents = true;
    this.doScripts = false;

    this.getKey = function()
    {
        return this.value;
    }

    this.setValue = function(value)
    {
        if (value) this.value = "" + value;
    }

    this.getValue = function()
    {
        return this.value;
    }

    this.setDoElements = function(value)
    {
        this.doElements = "" + value == "true";
    }

    this.getDoElements = function()
    {
        return this.doElements;
    }

    this.setDoContents = function(value)
    {
        this.doContents = "" + value != "false";
    }

    this.getDoContents = function()
    {
        return this.doContents;
    }

    this.setDoScripts = function(value)
    {
        this.doScripts = "" + value == "true";
    }

    this.getDoScripts = function()
    {
        return this.doScripts;
    }

    this.clone = function()
    {
        var tmp = new this.constructor(this.value, this.doElements, this.doContents, this.doScripts);
        for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(value|doElements|doContents|doScripts)$/.test(k)) tmp[k] = this[k];
        return tmp;
    }

    this.merge = function(o)
    {
        if (!o) return;
        this.setValue(o.getValue() || this.getValue());
        this.setDoElements(o.getDoElements() || this.getDoElements());
        this.setDoContents(o.getDoContents() && this.getDoContents());
        this.setDoScripts(o.getDoScripts() || this.getDoScripts());
    }

    this.generateXml = function()
    {
        if (!this.value) return "";
        return "\t\t\t" +
            "<pageReference" +
                " doElements=\"" + this.doElements + "\"" +
                " doContents=\"" + this.doContents + "\"" +
                " doScripts=\"" + this.doScripts + "\"" +
            ">" + this.value + "</pageReference>\n";
    }

    this.setValue(value);
    this.setDoElements(doElements);
    this.setDoContents(doContents);
    this.setDoScripts(doScripts);
}

